@@ -24,41 +24,50 @@ from collections import defaultdict
 import logging
 import os
 
-from guessit import PY2, u, guess_file_info
-from guessit.options import option_parser
+from guessit import PY2, u, guess_file_info, __version__
+from guessit.plugins import transformers
+from guessit.options import get_opts
+from guessit.__version__ import __version__
 
 
 def guess_file(filename, info='filename', options=None, **kwargs):
     options = options or {}
     filename = u(filename)
 
-    print('For:', filename)
+    if not options.get('yaml') and not options.get('show_property'):
+        print('For:', filename)
     guess = guess_file_info(filename, info, options, **kwargs)
-    if options.get('yaml'):
+
+    if not options.get('unidentified'):
         try:
-            import yaml
-            for k, v in guess.items():
-                if isinstance(v, list) and len(v) == 1:
-                    guess[k] = v[0]
-            ystr = yaml.safe_dump({filename: dict(guess)}, default_flow_style=False)
-            i = 0
-            for yline in ystr.splitlines():
-                if i == 0:
-                    print("? " + yline[:-1])
-                elif i == 1:
-                    print(":" + yline[1:])
-                else:
-                    print(yline)
-                i = i + 1
-            return
-        except ImportError:  # pragma: no cover
-            print('PyYAML not found. Using default output.')
+            del guess['unidentified']
+        except KeyError:
+            pass
+
+    if options.get('show_property'):
+        print (guess.get(options.get('show_property'), ''))
+        return
+
+    if options.get('yaml'):
+        import yaml
+        for k, v in guess.items():
+            if isinstance(v, list) and len(v) == 1:
+                guess[k] = v[0]
+        ystr = yaml.safe_dump({filename: dict(guess)}, default_flow_style=False)
+        i = 0
+        for yline in ystr.splitlines():
+            if i == 0:
+                print("? " + yline[:-1])
+            elif i == 1:
+                print(":" + yline[1:])
+            else:
+                print(yline)
+            i = i + 1
+        return
     print('GuessIt found:', guess.nice_string(options.get('advanced')))
 
 
 def _supported_properties():
-    from guessit.plugins import transformers
-
     all_properties = defaultdict(list)
     transformers_properties = []
     for transformer in transformers.all_transformers():
@@ -163,12 +172,17 @@ def run_demo(episodes=True, movies=True, options=None):
             print('-' * 80)
             guess_file(f, options=options, type='movie')
 
-def submit_bug(filename):
+def submit_bug(filename, options):
     import requests # only import when needed
     from requests.exceptions import RequestException
 
     try:
-        r = requests.post('http://guessit.io/bugs', {'filename': filename})
+        opts = dict((k, v) for k, v in options.__dict__.items()
+                    if v and k != 'submit_bug')
+
+        r = requests.post('http://localhost:5000/bugs', {'filename': filename,
+                                                         'version': __version__,
+                                                         'options': str(opts)})
         if r.status_code == 200:
             print('Successfully submitted file: %s' % r.text)
         else:
@@ -200,9 +214,9 @@ def main(args=None, setup_logging=True):
         sys.stdout = codecs.getwriter(locale.getpreferredencoding())(sys.stdout)
 
     if args:
-        options, args = option_parser.parse_args(args)
+        options, args = get_opts().parse_args(args)
     else:  # pragma: no cover
-        options, args = option_parser.parse_args()
+        options, args = get_opts().parse_args()
     if options.verbose:
         logging.getLogger().setLevel(logging.DEBUG)
 
@@ -217,20 +231,52 @@ def main(args=None, setup_logging=True):
     if options.demo:
         run_demo(episodes=True, movies=True, options=vars(options))
         help_required = False
-    elif options.submit_bug:
-        for filename in args:
-            help_required = False
-            submit_bug(filename)
-    else:
-        if args:
-            help_required = False
-            for filename in args:
+
+    if options.version:
+        print('+-------------------------------------------------------+')
+        print('+                   GuessIt ' + __version__ + (28-len(__version__)) * ' ' + '+')
+        print('+-------------------------------------------------------+')
+        print('|      Please report any bug or feature request at      |')
+        print('|       https://github.com/wackou/guessit/issues.       |')
+        print('+-------------------------------------------------------+')
+        help_required = False
+
+    if options.yaml:
+        try:
+            import yaml, babelfish
+            def default_representer(dumper, data):
+                return dumper.represent_str(str(data))
+            yaml.SafeDumper.add_representer(babelfish.Language, default_representer)
+            yaml.SafeDumper.add_representer(babelfish.Country, default_representer)
+        except ImportError:  # pragma: no cover
+            print('PyYAML not found. Using default output.')
+
+    filenames = []
+    if args:
+        filenames.extend(args)
+    if options.input_file:
+        input_file = open(options.input_file, 'r')
+        try:
+            filenames.extend([line.strip() for line in input_file.readlines()])
+        finally:
+            input_file.close()
+
+    filenames = filter(lambda filename: filename, filenames)
+
+    if filenames:
+        help_required = False
+        if options.submit_bug:
+            for filename in filenames:
+                submit_bug(filename, options)
+        else:
+            for filename in filenames:
                 guess_file(filename,
                            info=options.info.split(','),
                            options=vars(options))
 
+
     if help_required:  # pragma: no cover
-        option_parser.print_help()
+        get_opts().print_help()
 
 if __name__ == '__main__':
     main()
